import Joystick from '../../Texture/framework/joystick/Joystick';
import { SpeedType } from '../../Texture/framework/joystick/Joystick';
import GameData from '../GameData';
import Player from './Player';


// Learn TypeScript:
//  - [Chinese] https://docs.cocos.com/creator/manual/zh/scripting/typescript.html
//  - [English] http://www.cocos2d-x.org/docs/creator/manual/en/scripting/typescript.html
// Learn Attribute:
//  - [Chinese] https://docs.cocos.com/creator/manual/zh/scripting/reference/attributes.html
//  - [English] http://www.cocos2d-x.org/docs/creator/manual/en/scripting/reference/attributes.html
// Learn life-cycle callbacks:
//  - [Chinese] https://docs.cocos.com/creator/manual/zh/scripting/life-cycle-callbacks.html
//  - [English] http://www.cocos2d-x.org/docs/creator/manual/en/scripting/life-cycle-callbacks.html

const {ccclass, property} = cc._decorator;

@ccclass
export default class GameScene extends cc.Component {

    @property(cc.Camera)
    mainCamera: cc.Camera = null;

    @property(cc.Node)
    nodePlayer: cc.Node = null;

    @property(cc.TiledMap)
    tileMap: cc.TiledMap = null;

    @property(cc.TiledLayer)
    tileLayer: cc.TiledLayer = null;

    @property(cc.TiledObjectGroup)
    tileObj: cc.TiledObjectGroup = null;

    @property(Joystick)
    joystick: Joystick = null;

    _touchID: number = -1;

    // LIFE-CYCLE CALLBACKS:

    // onLoad () {}

    start () {
        // this.node.on("touchstart", this.onTouchStart, this);
        // this.node.on("touchmove", this.onTouchMove, this);
        // this.node.on("touchend", this.onTouchEnd, this);
        // this.node.on("touchcancel", this.onTouchCancel, this);

        cc.macro.FIX_ARTIFACTS_BY_STRECHING_TEXEL_TMX = 1;
        cc.macro.ENABLE_TILEDMAP_CULLING = true;

        GameData.player = this.nodePlayer;
        GameData.game = this;

        this.joystick.setCallback(this.movePlayer.bind(this));
    }

    //地图上的OpenGL坐标(左下角原点)转换为creator坐标(中心点原点)
    openGLToMap(pixel: cc.Vec2)
    {
        let mapSize = this.tileMap.getMapSize();
        let tileSize = this.tileMap.getTileSize();
        let x = pixel.x - mapSize.width*tileSize.width/2;
        let y = pixel.y - mapSize.height*tileSize.height/2;
        return cc.v2(x, y);
    }

    //地图上的creator坐标(中心点原点)转换为OpenGL坐标(左下角原点)
    mapToOpenGL(pixel: cc.Vec2)
    {
        let mapSize = this.tileMap.getMapSize();
        let tileSize = this.tileMap.getTileSize();
        let x = pixel.x + mapSize.width*tileSize.width/2;
        let y = pixel.y + mapSize.height*tileSize.height/2;
        return cc.v2(x, y);
    }

    //正常地图坐标转换
    //将像素坐标转换为tile坐标(左上角原点,向右向下递增)
    openGLToTile(pixel: cc.Vec2)
    {
        let mapSize = this.tileMap.getMapSize();
        let tileSize = this.tileMap.getTileSize();
        let x = Math.floor(pixel.x / tileSize.width);
        let y = Math.floor((mapSize.height*tileSize.height - pixel.y) / tileSize.height);
        return cc.v2(x, y);
    }

    //将tile坐标转换为像素坐标(格子中心)(左下角原点,向右向上递增)
    tileToOpenGL(tile: cc.Vec2)
    {
        let mapSize = this.tileMap.getMapSize();
        let tileSize = this.tileMap.getTileSize();
        let x = tile.x*tileSize.width + tileSize.width/2;
        let y = (mapSize.height-tile.y)*tileSize.height - tileSize.height/2;
        return cc.v2(x, y);
    }

    //45度及45度交错地图坐标转换
    //将像素坐标转换为tile坐标(左上角原点,向右向下递增)
    openGLToTile45(pixel)
    {
        let mapSize = this.tileMap.getMapSize();
        let tileSize = this.tileMap.getTileSize();
        let y = Math.floor((mapSize.height - 2 - ((2*Math.floor(pixel.y) / Math.floor(tileSize.height)))));
        let x = Math.floor(pixel.x/tileSize.width - (y%2) / 2);
        return cc.v2(x, y);
    }

    
    onDestroy()
    {
        // this.node.off("touchstart", this.onTouchStart, this);
        // this.node.off("touchmove", this.onTouchMove, this);
        // this.node.off("touchend", this.onTouchEnd, this);
        // this.node.off("touchcancel", this.onTouchCancel, this);
    }

    movePlayer(dir: cc.Vec2, speedType)
    {
        if (speedType == SpeedType.STOP) {
            
        } else {
            if (dir.equals(cc.Vec2.ZERO)) {
                return;
            }
            
            if (speedType == SpeedType.NORMAL) {
            } else if (speedType == SpeedType.FAST) {
            }

            this.nodePlayer.getComponent(Player).move(dir.mul(2));         
        }
    }

    onTouchStart(event: cc.Event.EventTouch)
    {   
        let touches:cc.Touch[] = event.getTouches();
        for (const touch of touches) {
            this._touchID = touch.getID();            
            return;
        }
    }

    onTouchMove(event: cc.Event.EventTouch)
    {
        let touches:cc.Touch[] = event.getTouches();
        for (const touch of touches) {
            if (this._touchID == touch.getID()) {
                let ptWorld = this.mainCamera.getScreenToWorldPoint(touch.getLocation());
                let point = this.node.convertToNodeSpaceAR(ptWorld);
                // let offset = this.nodePlayer.getPosition().sub(point);
                // this.nodePlayer.setPosition(this.nodePlayer.getPosition().add(offset)); 
                // this.nodePlayer.setPosition(point);
                let offset = touch.getDelta();
                this.nodePlayer.setPosition(this.nodePlayer.getPosition().add(offset));
                return;               
            }          
        }
    }

    onTouchEnd(event: cc.Event.EventTouch)
    {
    }
    
    onTouchCancel(event: cc.Event.EventTouch)
    {   
    }
}
