enum JoystickType 
{
	FOLLOW,
    FIXED,
}

export enum SpeedType 
{
    STOP,
	NORMAL,
	FAST,
}

let {ccclass, property} = cc._decorator;

@ccclass
export default class Joystick extends cc.Component {

	@property({type:cc.Node, tooltip:"摇杆操纵盘"})
	panel: cc.Node = null;

	@property({type:cc.Node, tooltip:"摇杆操纵点"})
    dot: cc.Node = null;
	
	@property({type:cc.Enum(JoystickType), tooltip:"摇杆类型"})
	joystickType: JoystickType = JoystickType.FOLLOW;
	
	//摇杆位置
	_panelLocation: cc.Vec2 = cc.v2(0,0);

	//触摸位置
	_touchLocation: cc.Vec2 = cc.v2(0,0);

	//速度类型
	_speedType: SpeedType = SpeedType.STOP;

	_radius = 0;

	_callback = null;

	_vector: cc.Vec2 = cc.Vec2.ZERO;

	onLoad()
	{
		this._radius = this.panel.width / 2;

		if (this.joystickType == JoystickType.FOLLOW) {
			this.node.opacity = 0;
		}

		this.node.on("touchstart", this.onTouchStart, this);
        this.node.on("touchmove", this.onTouchMove, this);
        this.node.on("touchend", this.onTouchEnd, this);
        this.node.on("touchcancel", this.onTouchEnd, this);
	}

	onDestroy()
    {
        this.node.off("touchstart", this.onTouchStart, this);
        this.node.off("touchmove", this.onTouchMove, this);
        this.node.off("touchend", this.onTouchEnd, this);
        this.node.off("touchcancel", this.onTouchEnd, this);
    }

	update(dt)
	{
		if (this._speedType != SpeedType.STOP) {
			this._callback(this._vector, this._speedType);
		}
	}

	//设置回调
	setCallback(callback)
	{
		this._callback = callback;
	}

	onTouchStart(event: cc.Event.EventTouch)
	{
		let touchLocation = this.node.convertToNodeSpaceAR(event.getLocation());

		if (this.joystickType === JoystickType.FIXED) {
			this._panelLocation = this.panel.getPosition();

			let distance = touchLocation.sub(this.panel.getPosition()).mag();
			if (this._radius > distance) {
				this.dot.setPosition(touchLocation);
			}

		} else if (this.joystickType === JoystickType.FOLLOW) {
			this._panelLocation = touchLocation;

			this.node.opacity = 255;
			this._touchLocation = event.getLocation();
			
			this.panel.setPosition(touchLocation);
			this.dot.setPosition(touchLocation);
		}
	}

	onTouchMove(event: cc.Event.EventTouch)
	{
		if (this.joystickType === JoystickType.FOLLOW) {
			if (this._touchLocation === event.getLocation()) {
				return false;
			}
		}
	
		let touchLocation = this.panel.convertToNodeSpaceAR(event.getLocation());
		let distance = touchLocation.mag();
	
		let point = this._panelLocation.add(touchLocation);
	
		this._vector = point.sub(this.panel.getPosition()).normalize();
	
		if (this._radius > distance) {
			this.dot.setPosition(point);

	  		this._speedType = SpeedType.NORMAL;
		} else {
			let x = this._panelLocation.x + this._vector.x * this._radius;
			let y = this._panelLocation.y + this._vector.y * this._radius;
			this.dot.setPosition(cc.v2(x, y));
		
			this._speedType = SpeedType.FAST;
		}
	}

	onTouchEnd(event: cc.Event.EventTouch)
	{
		this.dot.setPosition(this.panel.getPosition());
		if (this.joystickType == JoystickType.FOLLOW) {
		  	this.node.opacity = 0;
		}
		this._speedType = SpeedType.STOP;
		this._callback(this._vector, this._speedType);
	}
}